@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2011 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -154,6 +154,8 @@ typedef enum
 		state = JIS_UDC;	\
 	}
 
+static ConvADType conv_ad_type = C_AD_BY_LOCALE;
+
 static gchar *conv_jistoeuc(const gchar *inbuf, gint *error);
 static gchar *conv_jistosjis(const gchar *inbuf, gint *error);
 static gchar *conv_euctojis(const gchar *inbuf, gint *error);
@@ -768,13 +770,28 @@ static gchar *conv_jistoutf8(const gchar *inbuf, gint *error)
 	return utf8str;
 }
 
+#if USE_THREADS
+#define S_LOCK_DEFINE_STATIC(name)	G_LOCK_DEFINE_STATIC(name)
+#define S_LOCK(name)	G_LOCK(name)
+#define S_UNLOCK(name)	G_UNLOCK(name)
+#else
+#define S_LOCK_DEFINE_STATIC(name)
+#define S_LOCK(name)
+#define S_UNLOCK(name)
+#endif
+
 static gchar *conv_sjistoutf8(const gchar *inbuf, gint *error)
 {
 	static iconv_t cd = (iconv_t)-1;
 	static gboolean iconv_ok = TRUE;
+	S_LOCK_DEFINE_STATIC(cd);
+	gchar *ret;
+
+	S_LOCK(cd);
 
 	if (cd == (iconv_t)-1) {
 		if (!iconv_ok) {
+			S_UNLOCK(cd);
 			if (error)
 				*error = -1;
 			return g_strdup(inbuf);
@@ -787,6 +804,7 @@ static gchar *conv_sjistoutf8(const gchar *inbuf, gint *error)
 				g_warning("conv_sjistoutf8(): %s\n",
 					  g_strerror(errno));
 				iconv_ok = FALSE;
+				S_UNLOCK(cd);
 				if (error)
 					*error = -1;
 				return g_strdup(inbuf);
@@ -794,16 +812,23 @@ static gchar *conv_sjistoutf8(const gchar *inbuf, gint *error)
 		}
 	}
 
-	return conv_iconv_strdup_with_cd(inbuf, cd, error);
+	ret = conv_iconv_strdup_with_cd(inbuf, cd, error);
+	S_UNLOCK(cd);
+	return ret;
 }
 
 static gchar *conv_euctoutf8(const gchar *inbuf, gint *error)
 {
 	static iconv_t cd = (iconv_t)-1;
 	static gboolean iconv_ok = TRUE;
+	S_LOCK_DEFINE_STATIC(cd);
+	gchar *ret;
+
+	S_LOCK(cd);
 
 	if (cd == (iconv_t)-1) {
 		if (!iconv_ok) {
+			S_UNLOCK(cd);
 			if (error)
 				*error = -1;
 			return g_strdup(inbuf);
@@ -816,6 +841,7 @@ static gchar *conv_euctoutf8(const gchar *inbuf, gint *error)
 				g_warning("conv_euctoutf8(): %s\n",
 					  g_strerror(errno));
 				iconv_ok = FALSE;
+				S_UNLOCK(cd);
 				if (error)
 					*error = -1;
 				return g_strdup(inbuf);
@@ -823,7 +849,9 @@ static gchar *conv_euctoutf8(const gchar *inbuf, gint *error)
 		}
 	}
 
-	return conv_iconv_strdup_with_cd(inbuf, cd, error);
+	ret = conv_iconv_strdup_with_cd(inbuf, cd, error);
+	S_UNLOCK(cd);
+	return ret;
 }
 
 static gchar *conv_anytoutf8(const gchar *inbuf, gint *error)
@@ -852,9 +880,14 @@ static gchar *conv_utf8tosjis(const gchar *inbuf, gint *error)
 {
 	static iconv_t cd = (iconv_t)-1;
 	static gboolean iconv_ok = TRUE;
+	S_LOCK_DEFINE_STATIC(cd);
+	gchar *ret;
+
+	S_LOCK(cd);
 
 	if (cd == (iconv_t)-1) {
 		if (!iconv_ok) {
+			S_UNLOCK(cd);
 			if (error)
 				*error = -1;
 			return g_strdup(inbuf);
@@ -867,6 +900,7 @@ static gchar *conv_utf8tosjis(const gchar *inbuf, gint *error)
 				g_warning("conv_utf8tosjis(): %s\n",
 					  g_strerror(errno));
 				iconv_ok = FALSE;
+				S_UNLOCK(cd);
 				if (error)
 					*error = -1;
 				return g_strdup(inbuf);
@@ -876,16 +910,23 @@ static gchar *conv_utf8tosjis(const gchar *inbuf, gint *error)
 
 	if (isutf8bom(inbuf))
 		inbuf += 3;
-	return conv_iconv_strdup_with_cd(inbuf, cd, error);
+	ret = conv_iconv_strdup_with_cd(inbuf, cd, error);
+	S_UNLOCK(cd);
+	return ret;
 }
 
 static gchar *conv_utf8toeuc(const gchar *inbuf, gint *error)
 {
 	static iconv_t cd = (iconv_t)-1;
 	static gboolean iconv_ok = TRUE;
+	S_LOCK_DEFINE_STATIC(cd);
+	gchar *ret;
+
+	S_LOCK(cd);
 
 	if (cd == (iconv_t)-1) {
 		if (!iconv_ok) {
+			S_UNLOCK(cd);
 			if (error)
 				*error = -1;
 			return g_strdup(inbuf);
@@ -898,6 +939,7 @@ static gchar *conv_utf8toeuc(const gchar *inbuf, gint *error)
 				g_warning("conv_utf8toeuc(): %s\n",
 					  g_strerror(errno));
 				iconv_ok = FALSE;
+				S_UNLOCK(cd);
 				if (error)
 					*error = -1;
 				return g_strdup(inbuf);
@@ -907,7 +949,9 @@ static gchar *conv_utf8toeuc(const gchar *inbuf, gint *error)
 
 	if (isutf8bom(inbuf))
 		inbuf += 3;
-	return conv_iconv_strdup_with_cd(inbuf, cd, error);
+	ret = conv_iconv_strdup_with_cd(inbuf, cd, error);
+	S_UNLOCK(cd);
+	return ret;
 }
 
 static gchar *conv_utf8tojis(const gchar *inbuf, gint *error)
@@ -1373,7 +1417,8 @@ CodeConvFunc conv_get_code_conv_func(const gchar *src_encoding,
 
 	/* auto detection mode */
 	if (!src_encoding && !dest_encoding) {
-		if (conv_is_ja_locale())
+		if (conv_ad_type == C_AD_JAPANESE ||
+		    (conv_ad_type == C_AD_BY_LOCALE && conv_is_ja_locale()))
 			return conv_anytodisp;
 		else
 			return conv_noconv;
@@ -1651,6 +1696,8 @@ static const struct {
 	{"ja_JP.JIS"	, C_ISO_2022_JP	, C_ISO_2022_JP},
 #ifdef G_OS_WIN32
 	{"ja_JP"	, C_CP932	, C_ISO_2022_JP},
+#elif defined(__APPLE__)
+	{"ja_JP"	, C_UTF_8	, C_ISO_2022_JP},
 #else
 	{"ja_JP"	, C_EUC_JP	, C_ISO_2022_JP},
 #endif
@@ -1826,9 +1873,14 @@ static GHashTable *conv_get_charset_to_str_table(void)
 {
 	static GHashTable *table;
 	gint i;
+	S_LOCK_DEFINE_STATIC(table);
 
-	if (table)
+	S_LOCK(table);
+
+	if (table) {
+		S_UNLOCK(table);
 		return table;
+	}
 
 	table = g_hash_table_new(NULL, g_direct_equal);
 
@@ -1841,16 +1893,23 @@ static GHashTable *conv_get_charset_to_str_table(void)
 		}
 	}
 
+	S_UNLOCK(table);
 	return table;
 }
 
 static GHashTable *conv_get_charset_from_str_table(void)
 {
 	static GHashTable *table;
+	S_LOCK_DEFINE_STATIC(table);
+
 	gint i;
 
-	if (table)
+	S_LOCK(table);
+
+	if (table) {
+		S_UNLOCK(table);
 		return table;
+	}
 
 	table = g_hash_table_new(str_case_hash, str_case_equal);
 
@@ -1859,6 +1918,7 @@ static GHashTable *conv_get_charset_from_str_table(void)
 				    GUINT_TO_POINTER(charsets[i].charset));
 	}
 
+	S_UNLOCK(table);
 	return table;
 }
 
@@ -1885,32 +1945,41 @@ CharSet conv_get_locale_charset(void)
 	static CharSet cur_charset = -1;
 	const gchar *cur_locale;
 	const gchar *p;
-#ifndef G_OS_WIN32
+#if !defined(G_OS_WIN32) && !defined(__APPLE__)
 	gint i;
 #endif
+	S_LOCK_DEFINE_STATIC(cur_charset);
 
-	if (cur_charset != -1)
+	S_LOCK(cur_charset);
+
+	if (cur_charset != -1) {
+		S_UNLOCK(cur_charset);
 		return cur_charset;
+	}
 
 	cur_locale = conv_get_current_locale();
 	if (!cur_locale) {
 		cur_charset = C_US_ASCII;
+		S_UNLOCK(cur_charset);
 		return cur_charset;
 	}
 
-	if (strcasestr(cur_locale, "UTF-8")) {
+	if (strcasestr(cur_locale, "UTF-8") || strcasestr(cur_locale, "utf8")) {
 		cur_charset = C_UTF_8;
+		S_UNLOCK(cur_charset);
 		return cur_charset;
 	}
 
 	if ((p = strcasestr(cur_locale, "@euro")) && p[5] == '\0') {
 		cur_charset = C_ISO_8859_15;
+		S_UNLOCK(cur_charset);
 		return cur_charset;
 	}
 
-#ifdef G_OS_WIN32
+#if defined(G_OS_WIN32) || defined(__APPLE__)
 	cur_charset = conv_get_charset_from_str(conv_get_locale_charset_str());
 
+	S_UNLOCK(cur_charset);
 	return cur_charset;
 #else
 	for (i = 0; i < sizeof(locale_table) / sizeof(locale_table[0]); i++) {
@@ -1921,6 +1990,7 @@ CharSet conv_get_locale_charset(void)
 		if (!g_ascii_strncasecmp(cur_locale, locale_table[i].locale,
 					 strlen(locale_table[i].locale))) {
 			cur_charset = locale_table[i].charset;
+			S_UNLOCK(cur_charset);
 			return cur_charset;
 		} else if ((p = strchr(locale_table[i].locale, '_')) &&
 			 !strchr(p + 1, '.')) {
@@ -1928,12 +1998,14 @@ CharSet conv_get_locale_charset(void)
 			    !g_ascii_strncasecmp(cur_locale,
 						 locale_table[i].locale, 2)) {
 				cur_charset = locale_table[i].charset;
+				S_UNLOCK(cur_charset);
 				return cur_charset;
 			}
 		}
 	}
 
 	cur_charset = C_AUTO;
+	S_UNLOCK(cur_charset);
 	return cur_charset;
 #endif
 }
@@ -1941,9 +2013,12 @@ CharSet conv_get_locale_charset(void)
 const gchar *conv_get_locale_charset_str(void)
 {
 	static const gchar *codeset = NULL;
+	S_LOCK_DEFINE_STATIC(codeset);
+
+	S_LOCK(codeset);
 
 	if (!codeset) {
-#ifdef G_OS_WIN32
+#if defined(G_OS_WIN32) || defined(__APPLE__)
 		g_get_charset(&codeset);
 		if (!strcmp(codeset, CS_US_ASCII) ||
 		    !strcmp(codeset, CS_ANSI_X3_4_1968))
@@ -1953,7 +2028,13 @@ const gchar *conv_get_locale_charset_str(void)
 #endif
 	}
 
-	return codeset ? codeset : CS_INTERNAL;
+	if (codeset) {
+		S_UNLOCK(codeset);
+		return codeset;
+	}
+
+	S_UNLOCK(codeset);
+	return CS_INTERNAL;
 }
 
 CharSet conv_get_internal_charset(void)
@@ -1972,18 +2053,25 @@ CharSet conv_get_outgoing_charset(void)
 	const gchar *cur_locale;
 	const gchar *p;
 	gint i;
+	S_LOCK_DEFINE_STATIC(out_charset);
 
-	if (out_charset != -1)
+	S_LOCK(out_charset);
+
+	if (out_charset != -1) {
+		S_UNLOCK(out_charset);
 		return out_charset;
+	}
 
 	cur_locale = conv_get_current_locale();
 	if (!cur_locale) {
 		out_charset = C_AUTO;
+		S_UNLOCK(out_charset);
 		return out_charset;
 	}
 
 	if ((p = strcasestr(cur_locale, "@euro")) && p[5] == '\0') {
 		out_charset = C_ISO_8859_15;
+		S_UNLOCK(out_charset);
 		return out_charset;
 	}
 
@@ -2005,6 +2093,7 @@ CharSet conv_get_outgoing_charset(void)
 		}
 	}
 
+	S_UNLOCK(out_charset);
 	return out_charset;
 }
 
@@ -2048,6 +2137,9 @@ gboolean conv_is_multibyte_encoding(CharSet encoding)
 const gchar *conv_get_current_locale(void)
 {
 	static const gchar *cur_locale;
+	S_LOCK_DEFINE_STATIC(cur_locale);
+
+	S_LOCK(cur_locale);
 
 	if (!cur_locale) {
 #ifdef G_OS_WIN32
@@ -2058,14 +2150,17 @@ const gchar *conv_get_current_locale(void)
 			cur_locale = g_getenv("LC_CTYPE");
 		if (!cur_locale || *cur_locale == '\0')
 			cur_locale = g_getenv("LANG");
+#ifdef HAVE_LOCALE_H
 		if (!cur_locale || *cur_locale == '\0')
 			cur_locale = setlocale(LC_CTYPE, NULL);
+#endif /* HAVE_LOCALE_H */
 #endif /* G_OS_WIN32 */
 
 		debug_print("current locale: %s\n",
 			    cur_locale ? cur_locale : "(none)");
 	}
 
+	S_UNLOCK(cur_locale);
 	return cur_locale;
 }
 
@@ -2073,9 +2168,14 @@ gboolean conv_is_ja_locale(void)
 {
 	static gint is_ja_locale = -1;
 	const gchar *cur_locale;
+	S_LOCK_DEFINE_STATIC(is_ja_locale);
 
-	if (is_ja_locale != -1)
+	S_LOCK(is_ja_locale);
+
+	if (is_ja_locale != -1) {
+		S_UNLOCK(is_ja_locale);
 		return is_ja_locale != 0;
+	}
 
 	is_ja_locale = 0;
 	cur_locale = conv_get_current_locale();
@@ -2084,9 +2184,20 @@ gboolean conv_is_ja_locale(void)
 			is_ja_locale = 1;
 	}
 
+	S_UNLOCK(is_ja_locale);
 	return is_ja_locale != 0;
 }
 
+void conv_set_autodetect_type(ConvADType type)
+{
+	conv_ad_type = type;
+}
+
+ConvADType conv_get_autodetect_type(void)
+{
+	return conv_ad_type;
+}
+
 gchar *conv_unmime_header(const gchar *str, const gchar *default_encoding)
 {
 	gchar *buf;
@@ -2105,7 +2216,8 @@ gchar *conv_unmime_header(const gchar *str, const gchar *default_encoding)
 		}
 	}
 
-	if (conv_is_ja_locale())
+	if (conv_ad_type == C_AD_JAPANESE ||
+	    (conv_ad_type == C_AD_BY_LOCALE && conv_is_ja_locale()))
 		buf = conv_anytodisp(str, NULL);
 	else
 		buf = conv_localetodisp(str, NULL);
@@ -2160,20 +2272,22 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 
 	g_return_if_fail(g_utf8_validate(src, -1, NULL) == TRUE);
 
-	if (MB_CUR_MAX > 1) {
-		use_base64 = TRUE;
-		mimesep_enc = "?B?";
-	} else {
-		use_base64 = FALSE;
-		mimesep_enc = "?Q?";
-	}
-
 	src_encoding = CS_INTERNAL;
 	if (!out_encoding)
 		out_encoding = conv_get_outgoing_charset_str();
 	if (!strcmp(out_encoding, CS_US_ASCII))
 		out_encoding = CS_ISO_8859_1;
 
+	if (!g_ascii_strncasecmp(out_encoding, "ISO-8859-", 9) ||
+	    !g_ascii_strncasecmp(out_encoding, "KOI8-", 5) ||
+	    !g_ascii_strncasecmp(out_encoding, "Windows-", 8)) {
+		use_base64 = FALSE;
+		mimesep_enc = "?Q?";
+	} else {
+		use_base64 = TRUE;
+		mimesep_enc = "?B?";
+	}
+
 	mimestr_len = strlen(MIMESEP_BEGIN) + strlen(mimesep_enc) +
 		strlen(MIMESEP_END);
 
