@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2011 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -62,6 +62,11 @@ typedef struct _FolderItem	FolderItem;
 		  FOLDER_TYPE(FOLDER_ITEM(obj)->folder) == F_IMAP) &&	\
 		 !FOLDER_ITEM(obj)->no_select)
 
+#define FOLDER_ITEM_IS_SENT_FOLDER(obj)	((obj) &&			\
+					 ((obj)->stype == F_OUTBOX ||	\
+					  (obj)->stype == F_DRAFT ||	\
+					  (obj)->stype == F_QUEUE))
+
 typedef enum
 {
 	F_MH,
@@ -174,6 +179,15 @@ struct _FolderClass
 					 GSList		*file_list,
 					 gboolean	 remove_source,
 					 gint		*first);
+	gint     (*add_msg_msginfo)	(Folder		*folder,
+					 FolderItem	*dest,
+					 MsgInfo	*msginfo,
+					 gboolean	 remove_source);
+	gint     (*add_msgs_msginfo)	(Folder		*folder,
+					 FolderItem	*dest,
+					 GSList		*msginfo_list,
+					 gboolean	 remove_source,
+					 gint		*first);
 	gint     (*move_msg)		(Folder		*folder,
 					 FolderItem	*dest,
 					 MsgInfo	*msginfo);
@@ -227,6 +241,11 @@ struct _RemoteFolder
 	Folder folder;
 
 	Session *session;
+
+	guint remove_cache_on_destroy : 1;
+	guint dummy1 : 1;
+	guint dummy2 : 1;
+	guint dummy3 : 1;
 };
 
 #if 0
@@ -291,9 +310,11 @@ struct _FolderItem
 	gboolean trim_summary_subject;
 	gboolean trim_compose_subject;
 
+	GSList *cache_queue;
 	GSList *mark_queue;
 
 	guint last_selected;
+	gint qsearch_cond_type;
 
 	gpointer data;
 };
@@ -312,6 +333,12 @@ void        folder_destroy		(Folder		*folder);
 void        folder_local_folder_destroy	(LocalFolder	*lfolder);
 void        folder_remote_folder_destroy(RemoteFolder	*rfolder);
 
+gint        folder_remote_folder_destroy_all_sessions	(void);
+
+gboolean    folder_remote_folder_is_session_active
+					(RemoteFolder	*rfolder);
+gboolean folder_remote_folder_active_session_exist	(void);
+
 gint        folder_scan_tree		(Folder		*folder);
 gint        folder_create_tree		(Folder		*folder);
 
@@ -319,6 +346,7 @@ FolderItem *folder_item_new		(const gchar	*name,
 					 const gchar	*path);
 void        folder_item_append		(FolderItem	*parent,
 					 FolderItem	*item);
+FolderItem *folder_item_copy		(FolderItem	*item);
 void        folder_item_remove		(FolderItem	*item);
 void        folder_item_remove_children	(FolderItem	*item);
 void        folder_item_destroy		(FolderItem	*item);
@@ -351,6 +379,8 @@ FolderItem *folder_find_child_item_by_name	(FolderItem	*item,
 gchar      *folder_get_identifier		(Folder		*folder);
 gchar      *folder_item_get_identifier		(FolderItem	*item);
 FolderItem *folder_find_item_from_identifier	(const gchar	*identifier);
+FolderItem *folder_find_item_and_num_from_id	(const gchar	*identifier,
+						 gint		*num);
 
 Folder     *folder_get_default_folder	(void);
 FolderItem *folder_get_default_inbox	(void);
@@ -358,6 +388,11 @@ FolderItem *folder_get_default_outbox	(void);
 FolderItem *folder_get_default_draft	(void);
 FolderItem *folder_get_default_queue	(void);
 FolderItem *folder_get_default_trash	(void);
+FolderItem *folder_get_default_junk	(void);
+
+FolderItem *folder_get_junk		(Folder		*folder);
+void folder_set_junk			(Folder		*folder,
+					 FolderItem	*item);
 
 void folder_set_missing_folders		(void);
 void folder_unref_account_all		(PrefsAccount	*account);
@@ -386,6 +421,13 @@ gint   folder_item_add_msgs		(FolderItem	*dest,
 					 GSList		*file_list,
 					 gboolean	 remove_source,
 					 gint		*first);
+gint   folder_item_add_msg_msginfo	(FolderItem	*dest,
+					 MsgInfo	*msginfo,
+					 gboolean	 remove_source);
+gint   folder_item_add_msgs_msginfo	(FolderItem	*dest,
+					 GSList		*msginfo_list,
+					 gboolean	 remove_source,
+					 gint		*first);
 gint   folder_item_move_msg		(FolderItem	*dest,
 					 MsgInfo	*msginfo);
 gint   folder_item_move_msgs		(FolderItem	*dest,
