@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2010 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@ typedef struct _SummaryColumnState	SummaryColumnState;
 #include "folderview.h"
 #include "headerview.h"
 #include "messageview.h"
+#include "quick_search.h"
 #include "compose.h"
 #include "filter.h"
 #include "folder.h"
@@ -59,13 +60,6 @@ struct _SummaryView
 {
 	GtkWidget *vbox;
 
-	GtkWidget *search_hbox;
-	GtkWidget *filter_optmenu;
-	GtkWidget *filter_menu;
-	GtkWidget *search_label;
-	GtkWidget *search_entry;
-	GtkWidget *search_clear_btn;
-
 	GtkWidget *scrolledwin;
 	GtkWidget *treeview;
 
@@ -75,7 +69,6 @@ struct _SummaryView
 
 	GtkWidget *hseparator;
 	GtkWidget *hbox;
-	GtkWidget *hbox_l;
 	GtkWidget *statlabel_folder;
 	GtkWidget *statlabel_select;
 	GtkWidget *statlabel_msgs;
@@ -113,6 +106,7 @@ struct _SummaryView
 	FolderView  *folderview;
 	HeaderView  *headerview;
 	MessageView *messageview;
+	QuickSearch *qsearch;
 
 	FolderItem *folder_item;
 
@@ -163,6 +157,11 @@ private:
 
 	/* unthreading */
 	GSList *pos_list;
+
+	guint write_lock_count;
+
+	/* junk filter list */
+	GSList *junk_fltlist;
 };
 
 SummaryView	*summary_create(void);
@@ -174,9 +173,20 @@ gboolean summary_show		  (SummaryView		*summaryview,
 void summary_clear_list		  (SummaryView		*summaryview);
 void summary_clear_all		  (SummaryView		*summaryview);
 
+void summary_show_queued_msgs	  (SummaryView		*summaryview);
+
+/* full lock */
 void summary_lock		  (SummaryView		*summaryview);
 void summary_unlock		  (SummaryView		*summaryview);
 gboolean summary_is_locked	  (SummaryView		*summaryview);
+gboolean summary_is_read_locked	  (SummaryView		*summaryview);
+
+/* write lock (read only) */
+void summary_write_lock		  (SummaryView		*summaryview);
+void summary_write_unlock	  (SummaryView		*summaryview);
+gboolean summary_is_write_locked  (SummaryView		*summaryview);
+
+FolderItem *summary_get_current_folder		(SummaryView	*summaryview);
 
 SummarySelection summary_get_selection_type	(SummaryView	*summaryview);
 GSList *summary_get_selected_msg_list		(SummaryView	*summaryview);
@@ -273,6 +283,9 @@ void summary_select_all		  (SummaryView		*summaryview);
 void summary_unselect_all	  (SummaryView		*summaryview);
 void summary_select_thread	  (SummaryView		*summaryview);
 
+void summary_mark_displayed_read  (SummaryView		*summaryview,
+				   GtkTreeIter		*iter);
+
 void summary_reply		  (SummaryView		*summaryview,
 				   ComposeMode		 mode);
 
